/*
* common.js
* 自定义函数库
* wei.shi#shouqu.me
*/

/**
* 获取图片原始尺寸
* @params obj, 图像Element对象
* @params callback(res,obj), 回调函数，res:图片尺寸对象，obj:图片本身
*/
function getImgSize(obj,callback){
	var img = new Image();
	img.onload = function(){
		var res = {
			"width" : this.width,
			"height" : this.height
		};
		callback(res,obj);
	};
	img.src = obj.src;
}

/**
* 遍历DOM方法
*/
function traverseDOM (node, func) {
	func(node);
	node = node.firstChild;
	while (node) {
		traverseDOM(node, func);
		node = node.nextSibling;
	}
}

/**
* 获取元素数组by className属性
*/
function gEBCN(clsName){
	return document.getElementsByClassName(clsName);
}

/**
* 通过浏览器判断移动端平台类型
*/
function checkMobile(){
	var isMobile = {
		Android: function() {
			return !!navigator.userAgent.match(/Android/i) ? navigator.userAgent.match(/Android/i) : false;
		},
		BlackBerry: function() {
			return !!navigator.userAgent.match(/BlackBerry/i) ? navigator.userAgent.match(/BlackBerry/i) : false;
		},
		iOS: function() {
			return !!navigator.userAgent.match(/iPhone|iPad|iPod/i) ? navigator.userAgent.match(/iPhone|iPad|iPod/i) :false;
		},
		Opera: function() {
			return !!navigator.userAgent.match(/Opera Mini/i)?navigator.userAgent.match(/Opera Mini/i):false;
		},
		Windows: function() {
			return !!navigator.userAgent.match(/IEMobile/i)?navigator.userAgent.match(/IEMobile/i):false;
		},
		any: function() {
			return (isMobile.Android() || isMobile.BlackBerry() || isMobile.iOS() || isMobile.Opera() || isMobile.Windows());
		}
	};
	return isMobile.any();
}

/**
* 通过mediaquery 判断iphone类型
*/
function checkIphone(){
	var type = null;
	//media query
	var MediaQuery = {
		'iphone4':'only screen     and (min-device-width : 320px)    and (max-device-width : 480px)     and (orientation : portrait) 	and (color:8)    and (device-aspect-ratio : 320/480)        and (-webkit-device-pixel-ratio : 2)    and (-webkit-min-device-pixel-ratio : 2)',
		'iphone5':'only screen     and (min-device-width : 320px)    and (max-device-width : 568px)     and (orientation : portrait) 	and (color:8)    and (device-aspect-ratio : 320/568)        and (-webkit-device-pixel-ratio : 2)    and (-webkit-min-device-pixel-ratio : 2)',
		'iphone6':'only screen     and (min-device-width : 375px)    and (max-device-width : 667px)     and (orientation : portrait)     and (color : 8)    and (device-aspect-ratio : 375/667)        and (-webkit-device-pixel-ratio : 2)    and (-webkit-min-device-pixel-ratio : 2)',
		'iphone6plus':'only screen     and (min-device-width : 414px)     and (max-device-width : 736px)    and (device-width : 414px)    and (device-height : 736px)    and (orientation : portrait)     and (-webkit-min-device-pixel-ratio : 3)     and (-webkit-device-pixel-ratio : 3)'
	};
	for(var i in MediaQuery){
		if(window.matchMedia(MediaQuery[i]).matches){
			type = i;
			break;
		}
	}
	return type;
}

/** 
* 用户头像加载失败，加载默认头像图片 
* size 指定尺寸，默认图片为92x92
*/
function loadDefaultAvatar(size){
	var img = event.srcElement; 
	img.src = "http://shouqu.me/images/avatar@2x.png"; 
	img.onerror = null; //避免死循环
}

// ucs-2 string to base64 encoded ascii
function utoa(str) {
    return window.btoa(unescape(encodeURIComponent(str)));
}
// base64 encoded ascii to ucs-2 string
function atou(str) {
    return decodeURIComponent(escape(window.atob(str)));
}
/*
// Usage:
utoa('✓ à la mode'); // 4pyTIMOgIGxhIG1vZGU=
atou('4pyTIMOgIGxhIG1vZGU='); // "✓ à la mode"

utoa('I \u2661 Unicode!'); // SSDimaEgVW5pY29kZSE=
atou('SSDimaEgVW5pY29kZSE='); // "I ♡ Unicode!"
*/

//////////////////////笔记相关暴露给原生调用的全局函数////////////////////

//清除选中区域
function clearSelection(){
	var selection = window.getSelection();
	if(selection.anchorNode != null || selection.focusNode != null || selection.type.toLowerCase() != "none"){
		selection.collapseToStart();
		selection.removeAllRanges();
	}
}

//加载指定数据的笔记数据
function js_loadNoteData(data){
	//alert(JSON.stringify(data));
	//console.log("loading data",data);
	if(typeof data == "undefined"){
		console.log("数据值为空");
		return false;
	}
	if(typeof data != 'object'){
		console.log("数据格式有误");
		return false;
	}
	try{

		
		
		console.log("加载高亮数据...", data);

		//把回车转义替换为json兼容格式
		//data = data.replace(/\n/g,"\\n");
		//data = data.replace(/'/g,"\\'");
		/*
		data = data.replace(/\n|'/g,function(k){
			if( k.match(/\n/) ){
				return "\\n";
			}else if( k == "'" ){
				return "\\\\'";
			}
		});
		*/
		
		//如果有数据且没有版本号||有数据且版本号为1，则认为是旧版本数据
		//否则就是没有数据或者有数据且有版本号不为1
		//data不一定为null，具体值需要与app协商,app端需要传一个明确的值data.dataVersion为1或者2
		if(data!= null && ( typeof data.dataVersion == "undefined" || data.dataVersion == 1) ){
			MarkUtil.setDataFormatVersion(1);
			MarkUtil.formatTextNodes(1);
		}else{
			MarkUtil.setDataFormatVersion(2);
			MarkUtil.formatTextNodes(2);
		}
		
		//data本身是json object对象
		var noteData = data;
		window.currentNoteData = noteData;


		//window.bookmarkID = noteData.bookmarkID;//window存入bookmarkID
		//window.maxNoteIndex = bookmarkID + "_" + 1; //默认初始最大bookmark ID 为bookmarkID_1

		//notes数据不为空时才执行高亮标记
		if(noteData.notes && noteData.notes.length > 0){

			//根据已有notes数据计算最大可用noteIndex
			//var maxNoteTime = _.max(noteData.notes,function(o){return o.createTime});

			//需要根据note创建时间升序排序，然后依次在dom上标记高亮效果
			var sorted_notes = _.sortBy(noteData.notes,'createTime');
			noteData.notes = sorted_notes;

			MarkUtil.loadMarkData(noteData);
			//MarkUtil.loadMarkData(noteData);

			//根据note数据依次加高亮效果，并在每次高亮后，处理好起止节点未包括在高亮内的文字，将他们也包裹在sqt标签内，以保证下一次获取sqt 序号的正确性
		}

		//MarkUtil.loadMarkData(window.currentNoteData);

		//清除选中区域
		clearSelection();


	}catch(er){
		console.log(er.name + ":" + er.message);
		if(navigator.userAgent.match(/iPhone|iPad|iPod/i) == null && er.name.match(/IndexSizeError/ig) != null){
							console.log(data)
							//js_loadNoteData(data);
						}
	}

}

//highlight text and generate the selections data
function js_highlightNote(){

	//just for test 
	//console.log(typeof window.note);
	
	var selection = window.getSelection();

	if(selection.type.toLowerCase() == "none"){
		console.log("错误：没有选中任何区域");
	}else{
		//选区数据数组
		var selectionData;
		if(MarkUtil.dataFormatVersion == 1){
			selectionData = MarkUtil.getSelectionData(selection);
		}else if(MarkUtil.dataFormatVersion == 2){
			selectionData = MarkUtil.getSelectionDataV2(selection);
		}

		//保存到全局数据，供高亮回调函数使用
		window.selections = selectionData;

		//用于传给native的高亮数据对象
		var noteData = {
			selections : window.selections,
			highlightText : selection.toString(),
			dataVersion : MarkUtil.dataFormatVersion //增加数据格式版本字段
		};

		////console.log("发送给native保存...");
		try{
			if(typeof window.note != "undefined" && typeof window.note.native_saveNoteData != "undefined"){
				window.note.native_saveNoteData(JSON.stringify(noteData));
			}else{
				native_saveNoteData(noteData);
			}
		}catch(exception){
			console.log(exception.name + " : " + exception.message);
		}
	}

}

//native highlight callback
function js_highlightNoteCallback(noteID){
	if(typeof noteID == "number"){
		alert("传入的noteID为数字");
		return false;
	}
	console.log("传入的noteID:",noteID);
	if(typeof(noteID) != "undefined"){

		//组合高亮数据对象
		var noteData = {
			noteID : noteID+"",//强制转换为字符串型
			selections : window.selections,
			createTime: _.now()//js生成的时间戳，实际应用应当是app端生成并存入数据库
		};

		//用于MarkUtil.addMarks的参数是一个数组

		var notes = [noteData];
		//console.log("高亮回调：",noteData);

		//just for test
		// var note = {
		    // userId:"emulator",
		    // bookmarkId:"jidowkdi23432",
		    // notes:notes
		// };
		//console.log(JSON.stringify(note));

		// if(window.notes){
			// window.notes.push(noteData);
		// }

		//载入高亮数据
		MarkUtil.addMarks(notes);

		//clearSelection();

		//通知native加载
		try{
			if(typeof window.note != "undefined" && typeof window.note.native_reload != "undefined"){
				window.note.native_reload();
			}else if(typeof native_reload != "undefined"){
				native_reload();
			}
		}catch(exception){
			console.log(exception.name + " : " + exception.message);
		}

	}

}

//添加注释按钮
function js_addComment(noteID){

	if(typeof(noteID) != "undefined"){

		//拿到指定noteID的那部分高亮，给最后一个后面加上注释按钮
		var $note = $("." + MarkUtil.activeClassName + "[data-noteid='" + noteID + "']");
		if($note.last().next(".comment-icon").length == 0 ){

			$note.last().after('<i class="comment-icon" data-noteid="'+ noteID +'"><b>...</b></i>');

		}

	}else{

		//console.log("没有传入noteID");

	}

}

//删除高亮按钮并恢复原节点内容
function js_deleteNote(noteID){

	if(typeof(noteID) != "undefined"){

		//按noteID删除注释按钮
		//需要先删除注释按钮，否则已高亮部分会将按钮中的点作为文本加入到原先的文本中
		js_deleteComment(noteID);

		//高亮文本使用了<span class="mark-text">xxx</span>来标记文字，其父元素是span.MarkUtil.markSpanClassName
		//因此将父元素的html内容（包括未被高亮的文字部分）替换成其整个文本内容即可
		var $note = $("." + MarkUtil.activeClassName + "[data-noteid='" + noteID + "']");
		$note.each(function(i,v){
			//console.log(i);
			//$(v).removeClass(MarkUtil.activeClassName).removeAttr('data-noteid');
			var text = v.parentElement.innerText;
			var textNode = document.createTextNode(text);

			$(v.parentElement).empty().append(textNode);
		});

		//console.log("delete note done");

		//让选中区域消失
		js_collapseSelection();

		//通知native加载，上线版本中已关闭此命令
		//native_reload();

	}else{
		//console.log("没有传入noteID");
	}

}

//删除注释按钮
function js_deleteComment(noteID,commentID){

	if(typeof(noteID) != "undefined"){

		//使用jquery的remove方法将注释按钮删除
		$(".comment-icon[data-noteid='" + noteID + "']").remove();

		//通知native加载
		try{
			if(typeof window.note != "undefined" && typeof window.note.native_reload != "undefined" ){
				window.note.native_reload();
			}else if(typeof native_reload != "undefined"){
				native_reload();
			}
		}catch(exception){
			console.log(exception.name + " : " + exception.message);
		}

	}else{
		//console.log("没有传入noteID");
	}

}

//利用remove all ranges清除所有选中的文字
function js_collapseSelection(){

	var selection = window.getSelection();
	selection.removeAllRanges();

}

//获取选中部分文字
function js_getSelectionText(){
	var selection = window.getSelection();
	if(selection.type.toLowerCase() != "none"){
		return selection.toString();
	}
}

//书签页打开时跳转至指定ID元素所在位置
function scrollToElementByID(strElementID){

	//过滤空值参数
	if(typeof strElementID == "undefined"){
		console.log("错误：没有传入元素ID");
		return false;
	}

	//初始化高亮标记CSS类名，如果存在MarkUtil则使用MarkUtil中定义的类名
	var activeClassName = "mark-text";
	if(MarkUtil && MarkUtil.activeClassName){
		activeClassName = MarkUtil.activeClassName;
	}

	//根据ID属性或data-noteid属性获得元素
	var ele = document.getElementById(strElementID) == null ? document.querySelectorAll("." + MarkUtil.activeClassName + "[data-noteid='"+ strElementID +"']")[0] : document.getElementById(strElementID);

	//滚动到元素所在坐标
	window.scrollTo(ele.offsetLeft, ele.offsetTop);

}

//给body代理绑定高亮和注释按钮touchend事件，用mark-text和comment-icon两个类名称区分展示高亮或评论菜单
//并向navite传入用户点击的屏幕位置信息
function commentEventHandler(e){

	var screenx = e.changedTouches[0].clientX;
	var screeny = e.changedTouches[0].clientY;
	/*
	if(e.target.className != "" && e.target.className.match(/mark-text/)){
		var noteID = $(e.target).attr("data-noteid");
		////console.log(screenx,screeny,noteID);
		native_popMenushowWithLocation(screenx,screeny,noteID);

	}else*/ if( (e.target.className != "" && e.target.className.match(/comment-icon/) ) || ( e.target.tagName.toLowerCase() == "b" && e.target.parentElement.className.match(/comment-icon/) ) ){

		//获取点击对象的noteID,如果点击的是b标签则从其父元素获取noteID
		var noteID = $(e.target).attr("data-noteid");
		if(typeof noteID == "undefined"){
			noteID = $(e.target).parent(".comment-icon").attr("data-noteid");
		}

		//native_displayCommentMenu(screenx,screeny,noteID);
		try{
			if(typeof window.note != "undefined" && typeof window.note.native_displayCommentMenu != "undefined"){
				window.note.native_displayCommentMenu(screenx,screeny,noteID);;
			}else{
				native_displayCommentMenu(screenx,screeny,noteID);;
			}
		}catch(exception){
			//console.log(exception.name + " : " + exception.message);
		}

	}else{
		return false;
	}
}


//....////////////////////笔记相关暴露给原生调用的全局函数/////////////....///


